import Observable from "./observable.js";
import Signal from "./signal.js";
export default class AsyncIteratorFromCB {
    constructor() {
        this.#onClose = [];
        this.#onData = new Observable();
    }
    #onClose;
    #onData;
    onClose(callback) {
        this.#onClose.push(callback);
    }
    getCallback() {
        return (error, data) => {
            this.#onData.send({ error, data, close: undefined });
        };
    }
    send(data) {
        this.#onData.send({
            error: undefined,
            data,
            close: undefined,
        });
    }
    close() {
        this.#onData.send({
            close: true,
            data: undefined,
            error: undefined,
        });
        this.#onData.close();
        this.#onClose.forEach((cb) => cb());
    }
    [Symbol.asyncIterator]() {
        const queue = [];
        const signal = new Signal();
        this.#onData.subscribe((data) => {
            queue.push(data);
            signal.sendSignal();
        });
        return {
            async next() {
                const send = () => {
                    const value = queue.shift();
                    if (!value)
                        throw new Error("Error in AsyncIter");
                    if (value.close) {
                        return {
                            done: true,
                            value: undefined,
                        };
                    }
                    else if (value.error) {
                        throw value.error;
                    }
                    else {
                        return {
                            done: false,
                            value: value.data,
                        };
                    }
                };
                if (queue.length > 0) {
                    return send();
                }
                else {
                    await signal.awaitSignal();
                    return send();
                }
            },
        };
    }
}
//# sourceMappingURL=asynciter.js.map