/**
 * Basic Locking mechanism for JavaScript
 *
 */
export default class Lock {
    constructor() {
        this._locked = false;
        this.toCome = [];
        this.release = this.release.bind(this);
    }
    /**
     * Returns the state of the Locken
     *
     * @returns {boolean}
     */
    get locked() {
        return this._locked;
    }
    /**
     * Waits till lock is free and returns a release function
     *
     * @return {function}
     */
    async getLock() {
        if (!this._locked)
            return { release: this.lock() };
        else {
            return new Promise((resolve) => {
                this.toCome.push(() => {
                    resolve({ release: this.lock() });
                });
            });
        }
    }
    lock() {
        this._locked = true;
        return this.release;
    }
    async release() {
        let next = this.toCome.shift();
        if (next) {
            next();
        }
        else {
            this._locked = false;
        }
    }
}
//# sourceMappingURL=lock.js.map